//
//  serial_send.h
//
//  Created by kirara on 13/09/06.
//  Copyright (c) 2013年 kirara. All rights reserved.
//
//Cocoa/Objective-CとArduinoのシリアル通信
//http://ouipanda.hatenablog.com/entry/20091105/1257431329
//
//:: Arduino + Cocoa/Objective-C ::
//http://playground.arduino.cc/Interfacing/Cocoa
//
//Introduction to Device File Access Guide for Serial I/O
//https://developer.apple.com/library/mac/documentation/DeviceDrivers/Conceptual/WorkingWSerial/WWSerial_Intro/WWSerial_Intro.html
//
//Working With a Serial Device
//https://developer.apple.com/library/mac/documentation/DeviceDrivers/Conceptual/WorkingWSerial/WWSerial_SerialDevs/SerialDevices.html
//

#import <Cocoa/Cocoa.h>

// import IOKit headers
#include <IOKit/IOKitLib.h>
#include <IOKit/serial/IOSerialKeys.h>
#include <IOKit/IOBSD.h>
#include <IOKit/serial/ioss.h>
#include <sys/ioctl.h>

@interface AppDelegate : NSObject <NSApplicationDelegate>{

    NSWindow *window;

	int serialFileDescriptor;           // file handle to the serial port
	struct termios gOriginalTTYAttrs;   // Hold the original termios attributes so we can reset them on quit
                                        //  ( best practice )
	bool readThreadRunning;
    NSMutableString *receive_text;      // 受信した内容。
    NSMutableArray *step_array;         // 機種別ステップ数。

}

@property (strong, nonatomic) NSMutableString *receive_text;
@property (strong, nonatomic) NSMutableArray *step_array;

@property (assign) IBOutlet NSWindow *window;
@property (weak) IBOutlet NSPopUpButton *serialListPullDown;
@property (weak) IBOutlet NSPopUpButton *baudPopUpButton;
@property (weak) IBOutlet NSSlider *speed;
@property (weak) IBOutlet NSSegmentedControl *mode_select;
@property (weak) IBOutlet NSTextField *jikoku;
@property (weak) IBOutlet NSTextField *pos_gn;
@property (weak) IBOutlet NSTextField *pos_gp;
@property (weak) IBOutlet NSPopUpButton *count_select;
@property (weak) IBOutlet NSPanel *step_panel;
@property (weak) IBOutlet NSPanel *focuser_panel;

@property (weak) IBOutlet NSPopUpButton *move_select;
@property (weak) IBOutlet NSSegmentedControl *move_button;
@property (unsafe_unretained) IBOutlet NSTextView *step_view;
@property (unsafe_unretained) IBOutlet NSTextView *focuser_view;
@property (weak) IBOutlet NSMatrix *sync_radio;
@property (weak) IBOutlet NSButton *move_go_buttone;

- (IBAction)serialPortSelected:(id)sender;
- (IBAction)baudAction:(id)sender;
- (IBAction)speed_select:(id)sender;
- (IBAction)move_button:(id)sender;
- (IBAction)abort:(id)sender;
- (IBAction)step_panel_open:(id)sender;
- (IBAction)step_save:(id)sender;
- (IBAction)step_cansel:(id)sender;
- (IBAction)focuser_panel_open:(id)sender;
- (IBAction)focuser_save:(id)sender;
- (IBAction)focuser_cansel:(id)sender;
- (IBAction)move_select:(id)sender;
- (IBAction)sync_radio:(id)sender;
- (IBAction)move_go_buttone:(id)sender;

- (NSString *) openSerialPort: (NSString *)serialPortFile baud: (speed_t)baudRate;
- (void) refreshSerialList: (NSString *) selectedText;
- (void) writeString: (NSString *) str;

- (void)tick:(NSTimer *)theTimer;                           // timer
- (void)speed_send;                                         // speed send。
- (void)move_out;                                           //
- (void)move_in;                                            //
- (void)move_go;                                            //
- (void)move_sync;                                          //
- (void)focuser_load;                                       //
- (void)step_load;                                          //
- (void)sync_select;                                        //

@end
