//
//  serial_send.m
//
//  Created by kirara on 13/09/06.
//  Copyright (c) 2013年 kirara. All rights reserved.
//

#import "serial_send.h"

@implementation AppDelegate

// クローズボタンでアプリケーション終了。
- (BOOL)applicationShouldTerminateAfterLastWindowClosed:
(NSApplication *)theApplication
{

	return YES;
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Insert code here to initialize your application
    
	// we don't have a serial port open yet
	serialFileDescriptor = -1;
	readThreadRunning = FALSE;
    
	// first thing is to refresh the serial port list
	[self refreshSerialList:@"Select a Serial Port"];
    
    //Baudrate　PopUpButton
    [_baudPopUpButton removeAllItems];
    [_baudPopUpButton addItemsWithTitles:@[@"9600"]];
	[_baudPopUpButton selectItemWithTitle:@"9600"];

    [self focuser_load];
    [self step_load];
    [self sync_select];

    // timer
    NSTimer *timer;
    timer = [NSTimer scheduledTimerWithTimeInterval: 1
                                             target: self
                                             selector:@selector(tick:)
                                             userInfo: nil
                                             repeats: YES];
    
}

// open the serial port
//   - nil is returned on success
//   - an error message is returned otherwise

- (NSString *) openSerialPort: (NSString *)serialPortFile baud: (speed_t)baudRate {
	int success;
	
	// close the port if it is already open
	if (serialFileDescriptor != -1) {
		close(serialFileDescriptor);
		serialFileDescriptor = -1;
		
		// wait for the reading thread to die
		while(readThreadRunning);
		
		// re-opening the same port REALLY fast will fail spectacularly... better to sleep a sec
		sleep(0.5);
	}
	
	// c-string path to serial-port file
	const char *bsdPath = [serialPortFile cStringUsingEncoding:NSUTF8StringEncoding];
	
	// Hold the original termios attributes we are setting
	struct termios options;
	
	// receive latency ( in microseconds )
	unsigned long mics = 3;
	
	// error message string
	NSMutableString *errorMessage = nil;
	
	// open the port
	//     O_NONBLOCK causes the port to open without any delay (we'll block with another call)
	serialFileDescriptor = open(bsdPath, O_RDWR | O_NOCTTY | O_NONBLOCK );
	
	if (serialFileDescriptor == -1) {
		// check if the port opened correctly
		errorMessage = [NSMutableString stringWithString:@"Error: couldn't open serial port"];
	} else {
		// TIOCEXCL causes blocking of non-root processes on this serial-port
		success = ioctl(serialFileDescriptor, TIOCEXCL);
		if ( success == -1) {
			errorMessage = [NSMutableString stringWithString:@"Error: couldn't obtain lock on serial port"];
		} else {
			success = fcntl(serialFileDescriptor, F_SETFL, 0);
			if ( success == -1) {
				// clear the O_NONBLOCK flag; all calls from here on out are blocking for non-root processes
				errorMessage = [NSMutableString stringWithString:@"Error: couldn't obtain lock on serial port"];
			} else {
				// Get the current options and save them so we can restore the default settings later.
				success = tcgetattr(serialFileDescriptor, &gOriginalTTYAttrs);
				if ( success == -1) {
					errorMessage = [NSMutableString stringWithString:@"Error: couldn't get serial attributes"];
				} else {
					// copy the old termios settings into the current
					//   you want to do this so that you get all the control characters assigned
					options = gOriginalTTYAttrs;
					
					/*
					 cfmakeraw(&options) is equivilent to:
					 options->c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL | IXON);
					 options->c_oflag &= ~OPOST;
					 options->c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
					 options->c_cflag &= ~(CSIZE | PARENB);
					 options->c_cflag |= CS8;
					 */
					cfmakeraw(&options);
					
					// set tty attributes (raw-mode in this case)
					success = tcsetattr(serialFileDescriptor, TCSANOW, &options);
					if ( success == -1) {
						errorMessage = [NSMutableString stringWithString:@"Error: coudln't set serial attributes"];
					} else {
						// Set baud rate (any arbitrary baud rate can be set this way)
						success = ioctl(serialFileDescriptor, IOSSIOSPEED, &baudRate);
						if ( success == -1) {
							errorMessage = [NSMutableString stringWithString:@"Error: Baud Rate out of bounds"];
						} else {
							// Set the receive latency (a.k.a. don't wait to buffer data)
							success = ioctl(serialFileDescriptor, IOSSDATALAT, &mics);
							if ( success == -1) {
								errorMessage = [NSMutableString stringWithString:@"Error: coudln't set serial latency"];
							} else {
                                
                                NSLog(@"serial open");
                                _serialListPullDown.enabled  = NO;
                                _baudPopUpButton.enabled  = NO;
                                
                                //接続後のセットアップ。
                                [self setup_send];

                            }
						}
					}
				}
			}
		}
	}
	
	// make sure the port is closed if a problem happens
	if ((serialFileDescriptor != -1) && (errorMessage != nil)) {
		close(serialFileDescriptor);
		serialFileDescriptor = -1;
	}
	
	return errorMessage;
}

// This selector/function will be called as another thread...
//  this thread will read from the serial port and exits when the port is closed
- (void)incomingTextUpdateThread: (NSThread *) parentThread {
	
	// create a pool so we can use regular Cocoa stuff
	//   child threads can't re-use the parent's autorelease pool
    //	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	
	// mark that the thread is running
	readThreadRunning = TRUE;
	
	const   int BUFFER_SIZE = 30;
	char    byte_buffer[BUFFER_SIZE];       // buffer for holding incoming data
	int     numBytes=0;                     // number of bytes read during read
//    NSString *text;                         // incoming text from the serial port
	
	// assign a high priority to this thread
	[NSThread setThreadPriority:1.0];
	
	// this will loop unitl the serial port closes
	while(TRUE) {
        
        @autoreleasepool{
            
            // read() blocks until some data is available or the port is closed
            numBytes = read(serialFileDescriptor, byte_buffer, BUFFER_SIZE); // read up to the size of the buffer
            
            if(numBytes>0) {
                
                // create an NSString from the incoming bytes (the bytes aren't null terminated)
                _receive_text = [NSString stringWithCString:byte_buffer length:numBytes];
                NSLog(@"%@",_receive_text);
 
                if (_receive_text.length > 20) {
 
                } else {

                    NSArray *items = [_receive_text componentsSeparatedByString:@"#"];

                    // #GN
                    NSString *HEXString = items[0];
                    NSScanner* aScanner = [NSScanner scannerWithString: HEXString];

                    unsigned int uiValue;
                    [aScanner scanHexInt: &uiValue];
                    
                    NSString *str = [NSString stringWithFormat:@"%d",uiValue];
                    [_pos_gn setStringValue:str];
                    
                    NSLog(@"%d",uiValue);

                    // #GP
                    NSString *HEXString1 = items[1];
                    NSScanner* aScanner1 = [NSScanner scannerWithString: HEXString1];
                    
                    unsigned int uiValue1;
                    [aScanner1 scanHexInt: &uiValue1];
                    
                    NSString *str1 = [NSString stringWithFormat:@"%d",uiValue1];
                    [_pos_gp setStringValue:str1];
                    
                    NSLog(@"%d",uiValue1);

                }
                
                // this text can't be directly sent to the text area from this thread
                //  BUT, we can call a selctor on the main thread.
//              [self performSelectorOnMainThread:@selector(appendToIncomingText:)
//                                     withObject:text
//                                  waitUntilDone:YES];
                
                // you would want to do something useful here
//              NSLog([NSString stringWithCString:byte_buffer length:numBytes]);
                
            } else {
                break; // Stop the thread if there is an error
            }
        }
	}
	
	// make sure the serial port is closed
	if (serialFileDescriptor != -1) {
		close(serialFileDescriptor);
		serialFileDescriptor = -1;
	}
	
	// mark that the thread has quit
	readThreadRunning = FALSE;
	
	// give back the pool
    //	[pool release];
    
}

- (void) refreshSerialList: (NSString *) selectedText {
	io_object_t serialPort;
	io_iterator_t serialPortIterator;
	
	// remove everything from the pull down list
	[_serialListPullDown removeAllItems];
	
	// ask for all the serial ports
	IOServiceGetMatchingServices(kIOMasterPortDefault, IOServiceMatching(kIOSerialBSDServiceValue), &serialPortIterator);
	
	// loop through all the serial ports and add them to the array
	while (serialPort = IOIteratorNext(serialPortIterator)) {
		[_serialListPullDown addItemWithTitle:
         (__bridge_transfer NSString *)IORegistryEntryCreateCFProperty(serialPort, CFSTR(kIOCalloutDeviceKey),  kCFAllocatorDefault, 0)];
		IOObjectRelease(serialPort);
	}
    
	// add the selected text to the top
	[_serialListPullDown insertItemWithTitle:selectedText atIndex:0];
	[_serialListPullDown selectItemAtIndex:0];
	
	IOObjectRelease(serialPortIterator);
    
}

// send a string to the serial port
- (void) writeString: (NSString *) str {
	if(serialFileDescriptor!=-1) {
		write(serialFileDescriptor, [str cStringUsingEncoding:NSUTF8StringEncoding], [str length]);
	} else {
        // make sure the user knows they should select a serial port
//		[self appendToIncomingText:@"\n ERROR:  Select a Serial Port from the pull-down menu\n"];
	}
}

// action sent when serial port selected
- (IBAction) serialPortSelected: (id) cntrl {
	// open the serial port
	NSString *error = [self openSerialPort: [_serialListPullDown titleOfSelectedItem] baud:[[_baudPopUpButton titleOfSelectedItem]intValue]];
    
	if(error!=nil) {
		[self refreshSerialList:error];
//		[self appendToIncomingText:error];
	} else {
		[self refreshSerialList:[_serialListPullDown titleOfSelectedItem]];
		[self performSelectorInBackground:@selector(incomingTextUpdateThread:) withObject:[NSThread currentThread]];
	}
}

// action from baud rate change
- (IBAction) baudAction: (id) cntrl {
	if (serialFileDescriptor != -1) {
//		speed_t baudRate = [_baudInputField intValue];
		speed_t baudRate = [[_baudPopUpButton titleOfSelectedItem]intValue];
		
		// if the new baud rate isn't possible, refresh the serial list
		//   this will also deselect the current serial port
		if(ioctl(serialFileDescriptor, IOSSIOSPEED, &baudRate)==-1) {
			[self refreshSerialList:@"Error: Baud Rate out of bounds"];
		}
	}
}

// timer
- (void)tick:(NSTimer *)theTimer
{
    
    NSDate *date = [NSDate date];
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    
    /* 24時間表記 */

//  dateFormatter.dateFormat = @"yyyy/MM/dd HH:mm:ss";
    dateFormatter.dateFormat = @"HH:mm:ss";
    NSString *date24 = [dateFormatter stringFromDate:date];

    [_jikoku setStringValue:date24];

    // シリアルポート一覧取得。
    if (serialFileDescriptor != -1) {
    } else {
        // シリアルポート一覧取得。
        [self refreshSerialList:@"Select a Serial Port"];
        _serialListPullDown.enabled  = 1;
    }

 
    //
    [self writeString:@":GP#"];                 // Get Current Motor 1 Positon, Unsigned Hexadecimal
    [self writeString:@":GN#"];                 // Get the New Motor 1 Position, Unsigned Hexadecimal

   
}

// 接続後のコマンド送信。
-(void)setup_send
{

    NSLog(@"接続後のコマンド送信。");
  
    [NSThread sleepForTimeInterval:0.3];
    [self speed_send];
    
    NSLog(@"mode full");
    [NSThread sleepForTimeInterval:0.3];
    [self writeString:@":SF#"];
    
}

// speed select
- (IBAction)speed_select:(id)sender {
    
    [self speed_send];
    
}

// 速度。
-(void)speed_send
{

    [NSThread sleepForTimeInterval:0.5];

    float v;
    v = [_speed floatValue];
    
    int vInt = (int) v;
    
    NSLog(@"speed %d",vInt);
    
    switch (vInt) {
        case 5:
            [self writeString:@":SD20#"];
            break;
        case 4:
            [self writeString:@":SD10#"];
            break;
        case 3:
            [self writeString:@":SD08#"];
            break;
        case 2:
            [self writeString:@":SD04#"];
            break;
        case 1:
            [self writeString:@":SD02#"];
            break;
    }

}

// mode_select full/falf
- (IBAction)mode_select:(id)sender {
 
    [self speed_send];

    switch ([sender selectedSegment]) {

        case 0:
            // half
            [self writeString:@":SH#"];
            break;
            
        case 1:
            // full
            [self writeString:@":SF#"];
            break;
            
    }
    
}

// move IN/OUT
- (IBAction)move_button:(id)sender {
 
    [self speed_send];

    switch ([sender selectedSegment]) {
  
         // OUT
        case 0:
            
            [self move_out];
             break;
        
        // IN
        case 1:
            
            [self move_in];
            break;
            
    }
    
}

// ダイレクト移動。
- (IBAction)move_go_buttone:(id)sender {

    [self speed_send];

    NSLog(@"%@ が選択されました。",[_move_select titleOfSelectedItem]);

    NSInteger tag;
    
    tag = [[_sync_radio selectedCell] tag];
    
    switch(tag){
        case 0: // Move
            
            NSLog(@"Move。\n");
            [_move_go_buttone setTitle:@"GO"];
            [self move_go];

            break;
        case 1: // Sync
            
            NSLog(@"Sync。\n");
            [_move_go_buttone setTitle:@"Sync"];
            [self move_sync];
           
            break;
    }

}

// フォーカスアウト。
-(void)move_out
{
    
    [self writeString:@":FQ#"];             // abort

    int val_pos = [_pos_gn intValue];
    int count = [[_count_select titleOfSelectedItem] intValue];
    int set_count = count + val_pos;

    if (set_count > 65000) {
        set_count = 65000;
    }
    
    NSString *hex_count = [NSString stringWithFormat:@"%04X",set_count];
    NSLog(@"val_pos %d",val_pos);
    NSLog(@"set_count %d",set_count);
    NSLog(@"hex_count %@",hex_count);
    
    NSLog(@"%@ が選択されました。",[_count_select titleOfSelectedItem]);

    NSString *send_val = [NSString stringWithFormat:@":SN%@#",hex_count];

    [self writeString:send_val];
    [self writeString:@":FG#"];
 
}

// フォーカスイン。
-(void)move_in
{
  
    [self writeString:@":FQ#"];             // abort
    
    int val_pos = [_pos_gn intValue];
    int count = [[_count_select titleOfSelectedItem] intValue];
    int set_count = val_pos - count;
    
    if (set_count <= 0) {
        set_count = 0;
    }
    
    NSString *hex_count = [NSString stringWithFormat:@"%04X",set_count];
    NSLog(@"val_pos %d",val_pos);
    NSLog(@"set_count %d",set_count);
    NSLog(@"hex_count %@",hex_count);
    
    NSLog(@"%@ が選択されました。",[_count_select titleOfSelectedItem]);
    
    NSString *send_val = [NSString stringWithFormat:@":SN%@#",hex_count];
    
    [self writeString:send_val];
    [self writeString:@":FG#"];

}

// ダイレクト移動。
-(void)move_go
{
  
    long int array_id = [_move_select indexOfSelectedItem];
    NSLog(@"array_id %d",array_id);
    
    int step_val = [[_step_array objectAtIndex:array_id] intValue];
    NSLog(@"step_val %d",step_val);
    
    NSString *hex_count = [NSString stringWithFormat:@"%04X",step_val];
    NSLog(@"hex_count %@",hex_count);
    
    NSString *send_val = [NSString stringWithFormat:@":SN%@#",hex_count];
    
    [self writeString:send_val];
    [self writeString:@":FG#"];
    
}

// 同期。
-(void)move_sync
{
    
    long int array_id = [_move_select indexOfSelectedItem];
    NSLog(@"array_id %d",array_id);
    
    int step_val = [[_step_array objectAtIndex:array_id] intValue];
    NSLog(@"step_val %d",step_val);
    
    NSString *hex_count = [NSString stringWithFormat:@"%04X",step_val];
    NSLog(@"hex_count %@",hex_count);
    
    NSString *send_val = [NSString stringWithFormat:@":SN%@#",hex_count];
    [self writeString:send_val];

    [NSThread sleepForTimeInterval:0.3];

    NSString *send_val1 = [NSString stringWithFormat:@":SP%@#",hex_count];
    [self writeString:send_val1];

}

// abort
- (IBAction)abort:(id)sender {
    
    [self writeString:@":FQ#"];
    
    
    NSString *send_val = [NSString stringWithFormat:@":SP00FF#"];
    [self writeString:send_val];

    
}

// ダイレクト移動のデータの読込み。
- (void)focuser_load {
    
    // データの読込と表示。
    @autoreleasepool {
        
        // Resources内のファイルを読み込み。
        NSString *filePath = [[NSBundle mainBundle] pathForResource:@"focuser" ofType:@"csv"];
        NSString *text = [NSString stringWithContentsOfFile:filePath encoding:NSUTF8StringEncoding error:nil];
        
        // 改行文字で区切って配列に格納する
        NSArray *lines = [text componentsSeparatedByString:@"\n"];
        NSMutableArray *name_array = [NSMutableArray array];
        _step_array = [NSMutableArray array];

        int i = 0;
        
        for (NSString *row in lines) {
            
            //ブランク行以外。
            if (row.length > 0) {
                
                // コンマで区切って配列に格納する
                NSArray *items = [row componentsSeparatedByString:@","];
                [name_array addObject:items[0]];
                [_step_array addObject:items[1]];

                i++;

            }
        }
 
        [_move_select removeAllItems];
        [_move_select addItemsWithTitles:name_array];
        
        NSLog(@"name_array%@",[name_array description]);
        NSLog(@"step_array%@",[_step_array description]);
    
    }
}

// ステップ移動のデータの読込み。
- (void)step_load {
    
    // アプリケ−ションが実行された場所。
//  NSString *directoryPath = [[[NSBundle mainBundle] bundlePath] stringByDeletingLastPathComponent];
//  [[NSFileManager defaultManager] changeCurrentDirectoryPath:directoryPath];
    
    // データの読込と表示。
    @autoreleasepool {
        
        // UTF8 エンコードされた CSV ファイル
        //        NSString *filePath = [directoryPath stringByAppendingString:@"/step.csv"];
        //        NSString *text = [NSString stringWithContentsOfFile:filePath encoding:NSUTF8StringEncoding error:nil];
        
        // Resources内のファイルを読み込み。
        NSString *filePath = [[NSBundle mainBundle] pathForResource:@"step" ofType:@"csv"];
        NSString *text = [NSString stringWithContentsOfFile:filePath encoding:NSUTF8StringEncoding error:nil];
        
        // 改行文字で区切って配列に格納する
        NSArray *lines = [text componentsSeparatedByString:@"\n"];
        NSMutableArray *name_array = [NSMutableArray array];
        
        int i = 0;
        
        for (NSString *row in lines) {
            
            //ブランク行以外。
            if (row.length > 0) {
                
                // コンマで区切って配列に格納する
                NSArray *items = [row componentsSeparatedByString:@","];
                [name_array addObject:items[0]];
                
                i++;
                
            }
        }
        
        [_count_select removeAllItems];
        [_count_select addItemsWithTitles:name_array];
        
    }
}

- (IBAction)step_panel_open:(id)sender {
    
    [NSApp activateIgnoringOtherApps:YES];
    [_step_panel makeKeyAndOrderFront:self];
    
    NSString *filePath = [[NSBundle mainBundle] pathForResource:@"step" ofType:@"csv"];
    NSString *text = [NSString stringWithContentsOfFile:filePath encoding:NSUTF8StringEncoding error:nil];
        
    NSLog(@"text:%@",text);
    
    [_step_view setString:text];

}

- (IBAction)step_save:(id)sender {
    
    NSString *svString = [_step_view string];
    NSString *path = [[NSBundle mainBundle] pathForResource:@"step" ofType:@"csv"];
    [svString writeToFile: path atomically: YES encoding: NSUTF8StringEncoding error: NULL];

    [self step_load];

    [_step_panel close];

}

- (IBAction)step_cansel:(id)sender {
    
    [_step_panel close];

}

- (IBAction)focuser_panel_open:(id)sender {
    
    [NSApp activateIgnoringOtherApps:YES];
    [_focuser_panel makeKeyAndOrderFront:self];
    
    NSString *filePath = [[NSBundle mainBundle] pathForResource:@"focuser" ofType:@"csv"];
    NSString *text = [NSString stringWithContentsOfFile:filePath encoding:NSUTF8StringEncoding error:nil];
    
    NSLog(@"text:%@",text);
    
    [_focuser_view setString:text];
    
}

- (IBAction)focuser_save:(id)sender {
    
    NSString *svString = [_focuser_view string];
    NSString *path = [[NSBundle mainBundle] pathForResource:@"focuser" ofType:@"csv"];
    [svString writeToFile: path atomically: YES encoding: NSUTF8StringEncoding error: NULL];

    [self focuser_load];

    [_focuser_panel close];

}

- (IBAction)focuser_cansel:(id)sender {
    
    [_focuser_panel close];

}

- (IBAction)move_select:(id)sender {
    
    NSLog(@"%@ が選択されました。",[_move_select titleOfSelectedItem]);
    
}

- (IBAction)sync_radio:(id)sender {
 
    [self sync_select];

}

- (void)sync_select {

    NSInteger tag;
    
    tag = [[_sync_radio selectedCell] tag];
    
    switch(tag){
        case 0: // Move
            
            NSLog(@"Move。\n");
            [_move_go_buttone setTitle:@"GO"];
            
            break;
        case 1: // Sync
            
            NSLog(@"Sync。\n");
            [_move_go_buttone setTitle:@"Sync"];
            
            break;
    }
  
}

@end



//ラベル表示。
//[_lbl_rcv_ra  setStringValue:items[0]];

//表示している値を取得。
//int val_longitude_d = [_longitude_d intValue];

//変数に値をセット。
//int int_longitude_ss = [items2[1] intValue];
//NSString *longitude = [NSString stringWithFormat:@":Sr%02d:%02d:%02d#",int_longitude_hh,int_longitude_mm,int_longitude_ss];

//文字から数値変換。
//int val_lng_m = [_adv_ra_m intValue];

//数値から文字変換。
//NSString *str = [NSString stringWithFormat:@"%d",100];

//文字の比較。
//if ([hoge1 isEqualToString:hoge2]) {

